@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2015 Phusion Holding B.V.
+ *  Copyright (c) 2010-2016 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -42,9 +42,11 @@ typedef enum {
 
 typedef void PsgWatchdogLauncher;
 typedef void PsgVariantMap;
-typedef void (*PsgAfterForkCallback)(void *);
+typedef void (*PsgAfterForkCallback)(void *, void *);
 
 PsgVariantMap *psg_variant_map_new();
+char *psg_variant_map_get_optional(PsgVariantMap *m,
+	const char *name);
 void psg_variant_map_set(PsgVariantMap *m,
 	const char *name,
 	const char *value,
@@ -232,7 +234,7 @@ private:
 		}
 	}
 
-	void throwEnrichedWatchdogFailReason(const string &simpleReason) {
+	void throwEnrichedWatchdogFailReason(const ResourceLocator &locator, const string &simpleReason) {
 		if (mIntegrationMode == IM_STANDALONE) {
 			throw RuntimeException("Unable to start " PROGRAM_NAME ": " + simpleReason +
 				". This probably means that your " SHORT_PROGRAM_NAME
@@ -240,6 +242,7 @@ private:
 		} else {
 			string passengerRootConfig;
 			string docURL;
+
 			if (mIntegrationMode == IM_APACHE) {
 				passengerRootConfig = "PassengerRoot";
 				docURL = "https://www.phusionpassenger.com/library/config/apache/reference/#passengerroot";
@@ -247,11 +250,19 @@ private:
 				passengerRootConfig = "passenger_root";
 				docURL = "https://www.phusionpassenger.com/library/config/nginx/reference/#passenger_root";
 			}
-			throw RuntimeException("Unable to start " PROGRAM_NAME ": " + simpleReason +
-				". This probably means that your " SHORT_PROGRAM_NAME
-				" installation is broken or incomplete, or that your '" + passengerRootConfig +
-				"' setting contains the wrong value. Please reinstall " SHORT_PROGRAM_NAME
-				" or, or see " + docURL + " to learn how to adjust the setting.");
+
+			string message = "Unable to start " PROGRAM_NAME ": " + simpleReason +
+				". There may be different causes for this:\n\n"
+				" - Your '" + passengerRootConfig + "' setting is set to the wrong value."
+					" Please see " + docURL + " to learn how to fix the value.\n";
+			if (!locator.getBuildSystemDir().empty()) {
+				message.append(" - The " AGENT_EXE " binary is not compiled."
+					" Please run this command to compile it: "
+					+ locator.getBinDir() + "/passenger-config compile-agent\n");
+			}
+			message.append(" - Your " SHORT_PROGRAM_NAME " installation is broken or incomplete."
+				" Please reinstall " SHORT_PROGRAM_NAME ".");
+			throw RuntimeException(message);
 		}
 	}
 
@@ -269,7 +280,7 @@ private:
 	 * `timeout` miliseconds for the process to exit.
 	 */
 	static int timedWaitPid(pid_t pid, int *status, unsigned long long timeout) {
-		Timer timer;
+		Timer<SystemTime::GRAN_10MSEC> timer;
 		int ret;
 
 		do {
@@ -341,8 +352,7 @@ public:
 		try {
 			agentFilename = locator.findSupportBinary(AGENT_EXE);
 		} catch (const Passenger::RuntimeException &e) {
-			string locatorError = e.what();
-			throwEnrichedWatchdogFailReason(locatorError);
+			throwEnrichedWatchdogFailReason(locator, e.what());
 		}
 		SocketPair fds;
 		int e;
@@ -505,7 +515,7 @@ public:
 				killProcessGroupAndWait(&pid, 5000);
 				guard.clear();
 				if (e == ENOENT) {
-					throwEnrichedWatchdogFailReason("Executable " + agentFilename + " found.");
+					throwEnrichedWatchdogFailReason(locator, "Executable " + agentFilename + " not found.");
 				} else {
 					throw SystemException("Unable to start the " PROGRAM_NAME " watchdog (" +
 						agentFilename + ")", e);
