@@ -1,6 +1,6 @@
 # encoding: binary
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2010-2015 Phusion Holding B.V.
+#  Copyright (c) 2010-2016 Phusion Holding B.V.
 #
 #  "Passenger", "Phusion Passenger" and "Union Station" are registered
 #  trademarks of Phusion Holding B.V.
@@ -117,7 +117,7 @@ module PhusionPassenger
       else
         apache2ctl = self.apache2ctl
       end
-      if os_name == "linux" &&
+      if os_name_simple == "linux" &&
          linux_distro_tags.include?(:gentoo) &&
          apache2ctl == "/usr/sbin/apache2ctl"
         # On Gentoo, `apache2ctl -V` doesn't forward the command to `apache2 -V`,
@@ -582,7 +582,7 @@ module PhusionPassenger
         apxs2_flags = `#{apxs2} -q CFLAGS`.strip << " -I" << `#{apxs2} -q INCLUDEDIR`.strip
         apxs2_flags.gsub!(/-O\d? /, '')
 
-        if os_name =~ /solaris/
+        if os_name_simple == "solaris"
           if (language == :c && !cc_is_sun_studio?) || (language == :cxx && !cxx_is_sun_studio?)
             # Remove flags not supported by GCC
             # The big problem is Coolstack apxs includes a bunch of solaris -x directives.
@@ -597,7 +597,7 @@ module PhusionPassenger
           end
         end
 
-        if os_name == "linux" &&
+        if os_name_simple == "linux" &&
            linux_distro_tags.include?(:redhat) &&
            apxs2 == "/usr/sbin/apxs" &&
            httpd_architecture_bits == 64
@@ -608,10 +608,15 @@ module PhusionPassenger
           apxs2_flags.gsub!('-m32 -march=i386 -mtune=generic', '')
         end
 
+        # Some Apache installations include '-pie' in CFLAGS, which
+        # won't work on shared libraries.
+        # https://github.com/phusion/passenger/issues/1756
+        apxs2_flags.gsub!('-pie', '')
+
         apxs2_flags.strip!
         flags << apxs2_flags
       end
-      if !httpd.nil? && os_name == "macosx"
+      if !httpd.nil? && os_name_simple == "macosx"
         # The default Apache install on OS X is a universal binary.
         # Figure out which architectures it's compiled for and do the same
         # thing for mod_passenger. We use the 'file' utility to do this.
@@ -742,7 +747,7 @@ module PhusionPassenger
         flags = `#{apr_config} --cppflags --includes`.strip
         libs = `#{apr_config} --link-ld`.strip
         flags.gsub!(/-O\d? /, '')
-        if os_name =~ /solaris/
+        if os_name_simple == "solaris"
           if (language == :c && !cc_is_sun_studio?) || (language == :cxx && !cxx_is_sun_studio?)
             # Remove flags not supported by non-Sun Studio compilers
             flags = flags.split(/ +/).reject do |f|
@@ -750,7 +755,7 @@ module PhusionPassenger
             end
             flags = flags.join(' ')
           end
-        elsif os_name =~ /aix/
+        elsif os_name_simple == "aix"
           libs << " -Wl,-G -Wl,-brtl"
         end
         [flags, libs]
@@ -777,7 +782,7 @@ module PhusionPassenger
         flags = `#{apu_config} --includes`.strip
         libs = `#{apu_config} --link-ld`.strip
         flags.gsub!(/-O\d? /, '')
-        if os_name =~ /solaris/
+        if os_name_simple == "solaris"
           if (language == :c && !cc_is_sun_studio?) || (language == :cxx && !cxx_is_sun_studio?)
             # Remove flags not supported by non-Sun Studio compilers
             flags = flags.split(/ +/).reject do |f|
